# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
#
# Copyright © 2011 Stephen Baumgartner
#
# Permission to use, copy, modify, and distribute this software for
# any purpose and without fee is hereby granted, provided the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#-----------------------------------------------------------------------------
# Name        : tozero.rb
# Author      : Steve Baumgartner
# Email       : <steve@slbaumgartner.com>
# Website     : http://www.slbaumgartner.com
# Maintenance : Please report all bugs or strange behavior to the above
#             : Email address.
# Type        : Context Menu Items
# Description : Move selection items to Layer0
# Menu Item   : none
# Context-Menu: Primitives to Layer0, Explode to Layer0
# Date        : February 6,2012
# Version     :	1.0  - Initial Release.
#             : 1.1 - Use assignment of nil to layer= instead of looking up
#             : "Layer0".  This avoids dependency on the localized name of the
#             : layer and also the assumption that it is [0] in the layers array.
#             : 1.2 - Changed from modules to classes to keep methods out of
#             : the global namespace.  Changed to UI.Command implementation for
#             : better future flexibility.
#             : 1.3 -  Sept 7, 2013 - Enclose classes in my module.
# Usage       : Make a selection that includes one or more Groups and/or
#             : ComponentInstances.  On right-click, the context menu will
#             : include items "Primitives to Layer0" and "Explode to Layer0".
#             : - Primitives to Layer0 will probe recursively into all Groups and 
#             : ComponentInstances in the selection and will move all of
#             : their primitive Edges and Faces to Layer0.  It will leave the
#             : Groups and ComponentInstances on their original layers.  This
#             : will "heal" many situations in which the user has accidentally
#             : violated the guidance to draw everything on Layer0, especially
#             : by using the built-in explode, which puts the pieces on the 
#             : container's original layer.
#             : - Explode to Layer0 will explode each Group or ComponentInstance
#             : in the selection and will move all resulting Faces and Edges to
#             : Layer0.  Unlike Primitives to Layer0, it will not recursively
#             : probe into nested Groups or ComponentInstances to reassign the
#             : layers of their contents.
#-----------------------------------------------------------------------------

require 'sketchup.rb'

module SLBPlugins

  class SelectionToZero
    
    def SelectionToZero.selection_to_zero
      model = Sketchup.active_model
      @comp_defs = Hash.new
      # wrapping as an operation lets undo handle the entire manipulation as a unit
      # instead of backing out one primitive at a time.  The second argument tells
      # SketchUp to suspend updating the display during the operation, which can
      # make it run faster if there are a lot of primitives in the selection.
      model.start_operation "Primitives to Layer0", true
      sel = model.selection
      sel.each { |item|
        contents_to_zero(item)
      }
      model.commit_operation
      @comp_defs = nil
    end
    
    # recursive method to move primitive elements to Layer0
    def SelectionToZero.contents_to_zero( item )
      if(item.instance_of? Sketchup::Group)
        # The following call to make_unique will cause a warning messsage
        # on the Ruby Console saying that the method is deprecated.  However,
        # it appears necessary to use this method to work around a bug in
        # SketchUp's handling of Groups.
        defn = item.entities.parent
        item.make_unique if (defn.instances[1] != nil)
        item.entities.each { |subitem|
          contents_to_zero(subitem)
        }
      elsif(item.instance_of? Sketchup::ComponentInstance)
        # For efficiency sake, once we process a definition, we should not
        # bother doing it again if there is another instance of the same in the
        # selection.
        if(not @comp_defs.has_key?(item.definition) )
          @comp_defs[item.definition] = true
          item.definition.entities.each { |subitem|
            contents_to_zero(subitem)
          }
        end
      elsif(item.respond_to? "layer=")
        # not a container - move it to Layer0 if possible
        item.layer = nil
      else
        # entity does not accept a layer - nothing to do
      end
    end
  end #SelectionToZero

  class ExplodeToZero
    def ExplodeToZero.explode_to_zero
      model = Sketchup.active_model
      # wrapping as an operation lets undo handle the entire manipulation as a unit
      # instead of backing out one primitive at a time.  The second argument tells
      # SketchUp to suspend updating the display during the operation, which can make
      # it run faster when there are a lot of elements to process.
      model.start_operation "Explode to Layer0", true
      sel = model.selection
      sel.each { |item|
        # try to explode only entities that can be exploded!
        if (item.respond_to?("explode"))
          ents = item.explode
          # explode will have put all the entities onto the container's
          # layer, regardless of where they originally lived.  Reassign
          # them all to Layer0
          ents.each { |subitem|
            # skip reassignment for nested containers
            if(not (subitem.instance_of? Sketchup::Group or 
                    subitem.instance_of? Sketchup::ComponentInstance) )
              if(subitem.respond_to? "layer=")
                subitem.layer = nil
                # layer= also removes the subitem from the selection! 
                # put it back for consistency with standard explode
                sel.add(subitem)
              end
            end
          }
        else
          # not an explodable item.  Don't know why the user selected it,
          # but for consistency's sake move it to layer 0 also.
          if(item.respond_to? "layer=")
            item.layer = nil
            sel.add(item)
          end
        end
      }
      # mark the end of the undo-able operation
      model.commit_operation
    end
  end #ExplodeToZero

end # SLBPlugins

if( not file_loaded?(__FILE__) )
  ptz_cmd = UI::Command.new("底层线面 至 Layer0") {SLBPlugins::SelectionToZero.selection_to_zero}
  etz_cmd = UI::Command.new("炸开 至 Layer0") {SLBPlugins::ExplodeToZero.explode_to_zero}

  path = Sketchup.find_support_file("Plugins/SB_tozero")
  SB_tozero_tb = UI::Toolbar.new("修复至0层")
  cmd1 = UI::Command.new("底层线面 至 Layer0") {SLBPlugins::SelectionToZero.selection_to_zero}
  cmd1.small_icon = File.join(path,"icons/NZ024.png")
  cmd1.large_icon = File.join(path,"icons/NZ036.png")
  cmd1.tooltip = "底层线面 至 Layer0"
  cmd1.status_bar_text = "保持组及组件的属性，将其内部点线面归至layer 0 图层"
  SB_tozero_tb.add_item(cmd1)
    cmd2 = UI::Command.new("炸开 至 Layer0") {SLBPlugins::ExplodeToZero.explode_to_zero}
  cmd2.small_icon = File.join(path,"icons/ZZ024.png")
  cmd2.large_icon = File.join(path,"icons/ZZ036.png")
  cmd2.tooltip = "炸开 至 Layer0"
  cmd2.status_bar_text = "炸开组及组件，将炸开的结果归至layer 0 图层"
  SB_tozero_tb.add_item(cmd2)
  SB_tozero_tb.show

  # Every time the context menu is displayed, the UI invokes the code block
  # provided for each added context menu handler.  The code block can
  # decide whether the context is appropriate for this operation and
  # add the menu item or not.  The handler is "sticky" but the menu item
  # is not.  The handler must add it again the next time the context menu
  # is displayed.
  UI.add_context_menu_handler { |menu|
    # the target context is when something is selected that
    # can be exploded
    sel = Sketchup.active_model.selection
    if(not sel.empty?)
      # there is something selected - see if it is a target for these operations
      container = false
      sel.each { |sel_item|
        if(sel_item.instance_of? Sketchup::Group or 
           sel_item.instance_of? Sketchup::ComponentInstance)
          container = true
        end
      }
      if(container)
        # found a container in the selection - add our menu items
        menu.add_separator
        menu.add_item(ptz_cmd)
        menu.add_item(etz_cmd)
      end
    end
  }	
end
file_loaded(__FILE__)  
